* Class Browser Add-in to create a new class library
* containing subclasses of all classes in the current classlib
* Copyright: Public Domain.
* Written by: Tamar E. Granor and Jim Booth
* Original Date:   18-Dec-2000
* Version:  1.5 
 * Version Date:  9-Jan-2001

* Known issues:
*	- if a class being copied is subclassed from another class, and the 
*       reference to the parent class is incorrect (say, the parent classlib
*       has been moved), a dialog will appear to ask the user to find it.
*       Once the correct class library has been located, the user must
*       hit CTRL+W (or choose Save from the menu) to close the Class
*       Designer and proceed. This is because the initial error message
*       for the problem "eats" the keystroke intended to close the Class
*       Designer.

LPARAMETERS oBrowser

LOCAL nClass, lHasClass, loFrmCopyClass

* First, make sure the Browser is out there
IF VarType(oBrowser) <> "O"
	WAIT WINDOW "Start the Class Browser before running this program." NOWAIT
	RETURN .F.
ENDIF

WITH oBrowser
	* Now make sure there's a class library to work with
	IF .nClassCount = 0
		WAIT WINDOW "Open a class library in the Browser before running this program." NOWAIT
		RETURN .F.
	ENDIF

	* Make sure it's a classlib, not a form
	IF UPPER(JustExt(.cFileName)) <> "VCX"
		WAIT WINDOW "This program only works with class libraries." NOWAIT
		RETURN .F.
	ENDIF

	* Make sure this classlib has classes
	lHasClass = .F.
	FOR nClass = 1 TO .nClassCount
		IF .aClassList[nClass, 6] = .cFileName AND ;
		   STRTRAN(STRTRAN(.aClassList[nClass, 1], "(", ""), ")", "") <> .aClassList[ nClass, 6 ]
			* Found one
			lHasClass = .T.
			EXIT
		ENDIF
	ENDFOR
	
	IF NOT lHasClass
		WAIT WINDOW "The source library must contain at least one class" NOWAIT
		RETURN .F.
	ENDIF
ENDWITH	

* Display form here
loForm = CreateObject("frmCopyClassLib",oBrowser)
loForm.Show(1)
	
RETURN
	
**************************************************
* Class definitions
*
DEFINE CLASS frmCopyClassLib AS form


	Height = 248
	Width = 289
	DoCreate = .T.
	AutoCenter = .T.
	Caption = "Create Subclass Library"
	WindowType = 1
	Name = "frmCopyClassLib"
	obrowser = .F.


	ADD OBJECT pageframe1 AS pgfCopyClass
    ADD OBJECT cusFontHandler AS cusCodeFontHandler

	PROCEDURE setoptions
		* Enable or disable the prefix/suffix textboxes according to the Retain Original Names check box
		WITH Thisform.PageFrame1.Page1
			.txtNewPrefix.Enabled = NOT .chkRetainNames.Value
			.txtNewSuffix.Enabled = NOT .chkRetainNames.Value
			.txtOldPrefix.Enabled = NOT .chkRetainNames.Value
			.txtOldSuffix.Enabled = NOT .chkRetainNames.Value
		ENDWITH
	ENDPROC


	PROCEDURE copylibrary
		LOCAL lcNewLibrary, lcOldLibrary, lcNewClass, lcOldClass
		* Get name of the target library
		lcNewLibrary = ALLTRIM(Thisform.PageFrame1.Page1.txtDestinationLibrary.Value)
		* Get name of the source library
		lcOldLibrary = ALLTRIM(Thisform.PageFrame1.Page1.txtSourceLibrary.Value)

		* Check for any preexisting classes
		SELECT Preexists
		IF RECCOUNT() > 0
			* If any preexisting classes mark them according to the users choice
			SCAN FOR CopyOver
				SELECT Classes
				LOCATE FOR NewClassName = Preexists.NewClassName
				REPLACE CopyOver WITH .T.
			ENDSCAN
		ENDIF
		SELECT Classes
		IF NOT FILE(lcNewLibrary)
			* If the target library does not exist create it
			CREATE CLASSLIB (lcNewLibrary)
		ENDIF
		* Process the classes in the source library
		SCAN
			IF NOT Classes.Preexists OR (Classes.Preexists AND Classes.CopyOver)
				* If the class is new to the target or it is marked to be an over write
				IF Classes.Preexists
					* If this is an over write remove the existing class from the target
					REMOVE CLASS (ALLTRIM(Classes.NewClassName)) OF (lcNewLibrary)
				ENDIF
				* Set up to shut down the Class Designer
				KEYBOARD "{CTRL+F4}Y"
				* Create the class
				CREATE CLASS (ALLTRIM(Classes.NewClassName)) OF (lcNewLibrary) ;
				AS (ALLTRIM(Classes.OldClassName)) FROM (lcOldLibrary) 
			ENDIF
		ENDSCAN
		
		* Now open this library if requested
		IF THISFORM.Pageframe1.Page1.chkOpenLib.Value
			This.oBrowser.AddFile(lower(lcNewLibrary))
		ENDIF
		
		RETURN
	ENDPROC


	PROCEDURE getfilename
		* Displays the file selection dialog for selecting the target library
		LOCAL lcPath, lcFileName, lcNewFileName
		lcNewFileName = ""
		WITH Thisform.Pageframe1.Page1
			* Extract the path from the current file name
			lcPath = JustPath(.txtDestinationLibrary.Value)
			IF EMPTY( lcPath )
				* Add a backslash to the end if needed
				lcPath = ADDBS(CURDIR())
			ENDIF
			* Extract the file name without path or extension
			lcFileName = JustFName(.txtDestinationLibrary.Value)
			IF EMPTY( lcFileName )
				* Set default file name if the name was blank
				lcFileName = "Newlib"
			ENDIF
			* Produce the dialog
			lcNewFileName = PUTFILE("File Name:", lcFileName, "VCX")
			IF NOT EMPTY( lcNewFileName )
				* If a selection was made accept it
				.txtDestinationLibrary.Value = lcNewFileName
			ELSE
				* If no selection was made set to the default name
				.txtDestinationLibrary.Value = lcPath + lcFileName + ".VCX"
			ENDIF
		ENDWITH
		RETURN


	ENDPROC


	PROCEDURE detectdups
		* Searches target library for existing classes with duplicate names
		LOCAL nExisting, aExistingClasses[1], nMatches, aMatches[1], nMatchPos, lcNewLib, lcOldLib
		lcOldLib = ALLTRIM(Thisform.Pageframe1.Page1.txtSourceLibrary.Value)
		lcNewLib = ALLTRIM(Thisform.Pageframe1.Page1.txtDestinationLibrary.Value)
		IF NOT FILE( lcNewLib )
			* The the target library does not exist, then there is nothing to do here
			RETURN
		ENDIF
		SELECT Classes
		* Get a list of classes in the target library
		nExisting = AVCXClasses( aExistingClasses, lcNewLib )
		IF nExisting > 0
			* If there are any, compare existing classes to classes in source library
			nMatches = 0
			FOR nClass = 1 TO nExisting
				* Search the source new class names for a match
				LOCATE FOR UPPER(TRIM(NewClassName)) = UPPER(TRIM(aExistingClasses[ nClass, 1 ]))
				IF FOUND()
					* If we found a matching name, mark the Preexists Field of the Classes cursor
					REPLACE PreExists WITH .T.
					INSERT INTO Preexists (NewClassName) ;
					       VALUES (Classes.NewClassName)
				ENDIF
			ENDFOR
		ENDIF
		RETURN
	ENDPROC


	PROCEDURE validatecopy
		* Creates the new class names and posts them to the Classes Cursor
		* Calls either the copy method or the page with the grid for resolving the duplicate names
		LOCAL lcNewClassName, lcNewPreFix, lcOldPrefix, lcNewSuffix, lcOldSuffix
		SELECT Classes
		WITH Thisform.pageframe1.page1
			SCAN
				* Get the old class name in the existing library
				lcNewClassName = ALLTRIM(OldClassName)
				IF .chkRetainNames.Value
					* If no mods were requested, post the old name as the new and get the next one
					REPLACE NewClassName WITH lcNewClassName
					LOOP
				ENDIF
				* Get the name change requests
				lcNewPrefix = ALLTRIM(.txtNewPrefix.Value)
				lcNewSuffix = ALLTRIM(.txtNewSuffix.Value)
				lcOldPrefix = ALLTRIM(.txtOldPrefix.Value)
				lcOldSuffix = ALLTRIM(.txtOldSuffix.Value)
				IF NOT EMPTY( lcOldPreFix )
					* If a prefix is to be removed
					IF UPPER(LEFT(lcNewClassName,LEN(lcOldPrefix))) = UPPER(lcOldPrefix)
						* If that prefix is on this class name, remove it
						lcNewClassName = SUBSTR(lcNewClassName,LEN(lcOldPrefix)+1)
					ENDIF
				ENDIF
				IF NOT EMPTY( lcNewPreFix )
					* If a prefix is to be added, add it
					lcNewClassName = ALLTRIM(lcNewPrefix) + lcNewClassName
				ENDIF
				IF NOT EMPTY( lcOldSufFix )
					* If a suffix is to be removed
					IF UPPER(RIGHT(lcNewClassName,LEN(lcOldSuffix))) = UPPER(lcOldSuffix)
						* If that suffix is on this class name, remove it
						lcNewClassName = SUBSTR(lcNewClassName,1,LEN(lcNewClassName)-LEN(lcOldSuffix))
					ENDIF
				ENDIF
				IF NOT EMPTY( lcNewSufFix )
					* If a suffix is to be added, add it
					lcNewClassName = lcNewClassName + ALLTRIM(lcNewSuffix)
				ENDIF
				* Post the new name
				REPLACE NewClassName WITH lcNewClassName
			ENDSCAN
		ENDWITH
		* Detect any classes that may preexist in the destination library
		ThisForm.DetectDups()
		* Did any get found
		LOCATE FOR Preexists
		IF FOUND()
			* If yes, display the grid for overwrites
			ThisForm.Pageframe1.ActivePage = 2
			* Since the grid page does the copying if requested get out of here
			RETURN
		ENDIF
		* Do the actual copying of the classes
		Thisform.CopyLibrary()
		RETURN
	ENDPROC


	PROCEDURE Load
		* Create cursor for classes to be processed
		CREATE CURSOR Classes ( ;
			CopyOver L, ;
			OldClassName C(100), ;
			NewClassName C(100), ;
			PreExists L ;
			)

		* Create cursor for classes that already exist in the target library
		CREATE CURSOR Preexists ( ;
			CopyOver L, ;
			OldClassName C(100), ;
			NewClassName C(100), ;
			PreExists L ;
			)
	ENDPROC


	PROCEDURE Init
		LPARAMETERS toBrowser
		LOCAL lnCnt
        
        * Bump fonts for demo
        IF TYPE("glIsDemo")="L" AND glIsDemo
            This.cusFontHandler.ChangeFont(This,.t.)
            This.AutoCenter = .T.
        ENDIF
        
		WITH Thisform
			* Store reference to browser in form property
			.oBrowser = toBrowser
			WITH .PageFrame1.Page1
				* Display the name of the library to be copied
				.txtSourceLibrary.Value = ThisForm.oBrowser.cFileName
				* Display default new library file name
				.txtDestinationLibrary.Value = ADDBS(CURDIR()) + "NewLib.VCX"
			ENDWITH
			* Place each class name into the Classes Cursor
			FOR lnCnt = 2 TO ALEN( .oBrowser.aClassList,1)
				IF .oBrowser.aClassList(lnCnt,6) = .oBrowser.cFileName
					INSERT INTO Classes (OldClassName) ;
					     VALUES ( .oBrowser.aClassList(lnCnt,1))
				ENDIF
			ENDFOR
		ENDWITH
	ENDPROC


ENDDEFINE

DEFINE CLASS pgfCopyClass AS PageFrame

		ErasePage = .T.
		BorderWidth = 0
		Top = 0
		Left = 0
		Width = 291
		Height = 250
		Tabs = .F.
		Name = "Pageframe1"

		ADD OBJECT Page1 AS pagOne
		
		ADD OBJECT Page2 AS PagTwo

ENDDEFINE

DEFINE CLASS PagOne AS Page

	Caption = "Page1"
	Name = "Page1"

	ADD OBJECT txtsourcelibrary AS textbox WITH ;
		Height = 23, ;
		Left = 19, ;
		ReadOnly = .T., ;
		TabIndex = 1, ;
		Top = 18, ;
		Width = 229, ;
		Name = "txtSourceLibrary"

	ADD OBJECT label1 AS label WITH ;
		AutoSize = .T., ;
		Caption = "Class library to be copied:", ;
		Height = 17, ;
		Left = 19, ;
		Top = 2, ;
		Width = 141, ;
		TabIndex = 10, ;
		Name = "Label1"

	ADD OBJECT txtdestinationlibrary AS textbox WITH ;
		Height = 23, ;
		Left = 19, ;
		TabIndex = 2, ;
		Top = 61, ;
		Width = 229, ;
		Name = "txtDestinationLibrary"

	ADD OBJECT label2 AS label WITH ;
		AutoSize = .T., ;
		Caption = "Copy library to:", ;
		Height = 17, ;
		Left = 19, ;
		Top = 45, ;
		Width = 79, ;
		TabIndex = 11, ;
		Name = "Label2"

	ADD OBJECT cmdgetfilename AS cmdGetFileName

	ADD OBJECT chkretainnames AS chkRetainNames

	ADD OBJECT txtnewprefix AS textbox WITH ;
		Enabled = .F., ;
		Height = 23, ;
		InputMask = "XXXXX", ;
		Left = 97, ;
		TabIndex = 4, ;
		Top = 137, ;
		Width = 63, ;
		Name = "txtNewPrefix"

	ADD OBJECT txtnewsuffix AS textbox WITH ;
		Enabled = .F., ;
		Height = 23, ;
		InputMask = "XXXXX", ;
		Left = 97, ;
		TabIndex = 6, ;
		Top = 162, ;
		Width = 63, ;
		Name = "txtNewSuffix"

	ADD OBJECT label3 AS label WITH ;
		AutoSize = .T., ;
		Caption = "Prefix", ;
		Height = 17, ;
		Left = 61, ;
		Top = 140, ;
		Width = 32, ;
		TabIndex = 13, ;
		Name = "Label3"

	ADD OBJECT label4 AS label WITH ;
		AutoSize = .T., ;
		Caption = "Suffix", ;
		Height = 17, ;
		Left = 62, ;
		Top = 165, ;
		Width = 31, ;
		TabIndex = 14, ;
		Name = "Label4"
		
	ADD OBJECT chkOpenLib AS chkOpenLib

	ADD OBJECT cmdok AS cmdOKPage1

	ADD OBJECT cmdcancel AS cmdCancel WITH ;
		Top = 216, ;
		Left = 147, ;
		Height = 27, ;
		Width = 51

	ADD OBJECT label5 AS label WITH ;
		AutoSize = .T., ;
		Caption = "Add", ;
		Height = 17, ;
		Left = 117, ;
		Top = 121, ;
		Width = 23, ;
		TabIndex = 15, ;
		Name = "Label5"

	ADD OBJECT label6 AS label WITH ;
		AutoSize = .T., ;
		Caption = "Remove", ;
		Height = 17, ;
		Left = 173, ;
		Top = 121, ;
		Width = 48, ;
		TabIndex = 16, ;
		Name = "Label6"

	ADD OBJECT txtoldprefix AS textbox WITH ;
		Enabled = .F., ;
		Height = 23, ;
		InputMask = "XXXXX", ;
		Left = 166, ;
		TabIndex = 5, ;
		Top = 137, ;
		Width = 63, ;
		Name = "txtOldPrefix"

	ADD OBJECT txtoldsuffix AS textbox WITH ;
		Enabled = .F., ;
		Height = 23, ;
		InputMask = "XXXXX", ;
		Left = 166, ;
		TabIndex = 7, ;
		Top = 162, ;
		Width = 63, ;
		Name = "txtOldSuffix"

ENDDEFINE

DEFINE CLASS grdPreexists AS Grid

	DeleteMark = .F.
	Height = 162
	Left = 11
	Panel = 1
	RecordMark = .F.
	RecordSource = "Preexists"
	Top = 54
	Width = 269
	Name = "Grid1"

	ADD OBJECT Column1 AS Col1
	
	ADD OBJECT Column2 AS Col2
	
ENDDEFINE


DEFINE CLASS Col1 AS Column

	Name = "Column1"
	Sparse = .F.

	ADD OBJECT check1 AS checkbox WITH ;
		Top = 28, ;
		Left = 19, ;
		Height = 17, ;
		Width = 60, ;
		Alignment = 1, ;
		Caption = "", ;
		ControlSource = "Preexists.CopyOver", ;
		Name = "Check1"

	PROCEDURE Init
		This.Header1.Caption = "Overwrite?"
		This.CurrentControl = "Check1"
	ENDPROC
	
ENDDEFINE

DEFINE CLASS Col2 AS Column

	ControlSource = "Preexists.NewClassName"
	Width = 187
	Name = "Column2"

	PROCEDURE Init
		This.Header1.Caption = "Class name"
	ENDPROC
	
ENDDEFINE

DEFINE CLASS PagTwo AS Page

	ADD OBJECT Grid1 AS grdPreexists

	ADD OBJECT cmdok AS cmdOKPage2

	ADD OBJECT cmdcancel AS cmdCancel WITH ;
		Top = 219, ;
		Left = 147, ;
		Height = 27, ;
		Width = 51

	ADD OBJECT label1 AS label WITH ;
		WordWrap = .T., ;
		Caption = "The classes listed below already exist in the target library. Check a class to overwrite it with a new subclass. Uncheck it to keep the existing class.", ;
		Height = 46, ;
		Left = 5, ;
		Top = 0, ;
		Width = 280, ;
		Name = "Label1"

	PROCEDURE Activate
		This.Grid1.Refresh()
	ENDPROC

ENDDEFINE

DEFINE CLASS cmdGetFileName AS CommandButton

		Top = 61
		Left = 251
		Height = 23
		Width = 21
		FontBold = .T.
		FontSize = 10
		Caption = "..."
		TabIndex = 12
		Name = "cmdGetFileName"

	PROCEDURE Click
		Thisform.GetFileName()
	ENDPROC

ENDDEFINE


DEFINE CLASS chkRetainNames AS CheckBox

		Top = 99
		Left = 56
		Height = 17
		Width = 178
		AutoSize = .T.
		Caption = "Retain original class names"
		Value = .T.
		TabIndex = 3
		Name = "chkRetainNames"

	PROCEDURE InteractiveChange
		Thisform.SetOptions()
	ENDPROC

ENDDEFINE

DEFINE CLASS cmdOKPage1 AS CommandButton

		Top = 216
		Left = 93
		Height = 27
		Width = 51
		Caption = "OK"
		Default = .F.
		TabIndex = 8
		Name = "cmdOK"

	PROCEDURE Click
		Thisform.ValidateCopy()
		* If any dup names were detected the activepage will be 2 and 
		* page 2's buttons will handle the form's release
		IF Thisform.Pageframe1.ActivePage = 1
			Thisform.Release()
		ENDIF
	ENDPROC

ENDDEFINE

DEFINE CLASS cmdCancel AS CommandButton

		Cancel = .T.
		Caption = "Cancel"
		TabIndex = 9
		Name = "cmdCancel"

	PROCEDURE Click
		Thisform.Release()
	ENDPROC

ENDDEFINE

DEFINE CLASS cmdOKPage2 AS CommandButton

		Top = 219
		Left = 93
		Height = 27
		Width = 51
		Caption = "Copy"
		Default = .T.
		TabIndex = 8
		Name = "cmdOK"

	PROCEDURE Click
		* Validation has already been handled by page 1's button, 
		* so just copy the library
		Thisform.CopyLibrary()
		Thisform.Release()
	ENDPROC

ENDDEFINE

DEFINE CLASS chkOpenLib AS CheckBox

		Top = 192
		Left = 25
		Height = 17
		Width = 241
		Caption = "Open new class library in Class Browser"
		TabIndex = 8
		Value = .T.
		Name = "chkOpenLib"
		
ENDDEFINE
		
DEFINE CLASS cusCodeFontHandler AS cusFontHandler OF accessibility.vcx
* No changes needed
ENDDEFINE

